/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file stat.c
 * Implementation of the s3 DSI stat
 */

#include "globus_gridftp_server_s3_stat.h"

static
void
globus_l_s3_stat_mode_string(
    int                                 mode,
    char                                mode_string[static 11])
{
    size_t mode_index = 0;
    switch (mode & S_IFMT)
    {
        case S_IFIFO: mode_string[mode_index] = 'p'; break;
        case S_IFCHR: mode_string[mode_index] = 'c'; break;
        case S_IFDIR: mode_string[mode_index] = 'd'; break;
        case S_IFBLK: mode_string[mode_index] = 'b'; break;
        case S_IFREG: mode_string[mode_index] = '-'; break;
        case S_IFLNK: mode_string[mode_index] = 'l'; break;
        case S_IFSOCK: mode_string[mode_index] = 's'; break;
        default: mode_string[mode_index] = '*'; break;
    }
    mode_index++;

    /* User Read (or not) */
    if (mode & S_IRUSR)
    {
        mode_string[mode_index++] = 'r';
    }
    else
    {
        mode_string[mode_index++] = '-';
    }

    /* User write (or not) */
    if (mode & S_IWUSR)
    {
        mode_string[mode_index++] = 'w';
    }
    else
    {
        mode_string[mode_index++] = '-';
    }

    /* User execute and/or setuid or not */
    if (mode & S_IXUSR)
    {
        if (mode & S_ISUID)
        {
            mode_string[mode_index++] = 's';
        }
        else
        {
            mode_string[mode_index++] = 'x';
        }
    }
    else if (mode & S_ISUID)
    {
        mode_string[mode_index++] = 'S';
    }
    else
    {
        mode_string[mode_index++] = '-';
    }

    /* Group Read (or not) */
    if (mode & S_IRGRP)
    {
        mode_string[mode_index++] = 'r';
    }
    else
    {
        mode_string[mode_index++] = '-';
    }

    /* Group Write (or not) */
    if (mode & S_IWGRP)
    {
        mode_string[mode_index++] = 'r';
    }
    else
    {
        mode_string[mode_index++] = '-';
    }

    /* Group execute and/or setgid or not */
    if (mode & S_IXGRP)
    {
        if (mode & S_ISGID)
        {
            mode_string[mode_index++] = 's';
        }
        else
        {
            mode_string[mode_index++] = 'x';
        }
    }
    else if (mode & S_ISGID)
    {
        mode_string[mode_index++] = 'S';
    }
    else
    {
        mode_string[mode_index++] = '-';
    }
    /* Other Read (or not) */
    if (mode & S_IROTH)
    {
        mode_string[mode_index++] = 'r';
    }
    else
    {
        mode_string[mode_index++] = '-';
    }

    /* Other Write (or not) */
    if (mode & S_IWOTH)
    {
        mode_string[mode_index++] = 'r';
    }
    else
    {
        mode_string[mode_index++] = '-';
    }

    /* Other execute and/or sticky or not */
    if (mode & S_IXOTH)
    {
        if (mode & S_ISVTX)
        {
            mode_string[mode_index++] = 't';
        }
        else
        {
            mode_string[mode_index++] = 'x';
        }
    }
    else if (mode & S_ISGID)
    {
        mode_string[mode_index++] = 'T';
    }
    else
    {
        mode_string[mode_index++] = '-';
    }
    mode_string[mode_index++] = '\0';
}
/* globus_l_s3_stat_mode_string() */

void
globus_i_s3_stat_add(
    globus_i_s3_stat_t                 *stat_state,
    globus_gfs_stat_t                  *stat)
{
    GlobusS3Enter();
    char                                mode_string[11];

    stat_state->stat[stat_state->stat_count] = *stat;
    stat_state->stat[stat_state->stat_count++].name = strdup(stat->name);

    globus_l_s3_stat_mode_string(stat->mode, mode_string);

    GlobusS3Trace("Adding stat result for %s [mode=%s size=%"GLOBUS_OFF_T_FORMAT"]\n",
        stat->name,
        mode_string,
        stat->size);

    if (stat_state->stat_count == GLOBUS_S3_STAT_COUNT)
    {
        GlobusS3Debug("calling finished_stat_partial stat_count=%d\n",
                stat_state->stat_count);

        globus_gridftp_server_finished_stat_partial(
                stat_state->op,
                GLOBUS_SUCCESS,
                stat_state->stat,
                stat_state->stat_count);

        for (int i = 0; i < stat_state->stat_count; i++)
        {
            free(stat_state->stat[i].name);
            stat_state->stat[i] = (globus_gfs_stat_t){.name = NULL};
        }
        stat_state->stat_count = 0; 
    }
    GlobusS3Exit();
}
/* globus_i_s3_stat_add() */
// vim: ft=c:
